--[[
Script to fix the dashboard for trailer turn lights

Author:		Ifko[nator]
Date:		17.03.2024
Version:	1.0

History:	V1.0 @17.03.2024 - initial implementation in FS 22
]]

TrailerTurnLightDashboardFix = {};
TrailerTurnLightDashboardFix.currentModName = g_currentModName;

TrailerTurnLightDashboardFix.supportetTrailerTurnLights = {
	"TURN_LIGHT_TRAILER_ANY_1",
	"TURN_LIGHT_TRAILER_LEFT_1",
	"TURN_LIGHT_TRAILER_LEFT_2",
	"TURN_LIGHT_TRAILER_ANY_2",
	"TURN_LIGHT_TRAILER_RIGHT_1", 
	"TURN_LIGHT_TRAILER_RIGHT_2"
};

local function getSpecByName(self, specName, currentModName)
    local spec = self["spec_" .. Utils.getNoNil(currentModName, TrailerTurnLightDashboardFix.currentModName) .. "." .. specName];

	if spec ~= nil then
        return spec;
    end;

    return self["spec_" .. specName];
end;

local function getHasTurnLights(xmlFile)
	local hasTurnLights = false;

	for _, lightType in pairs({"low", "high"}) do
		for _, direction in pairs({"Left", "Right"}) do
			if xmlFile:hasProperty("vehicle.lights.realLights." .. lightType .. ".turnLight" .. direction) then
				hasTurnLights = true;

				break;
			end;
		end;
	end;

	return hasTurnLights;
end;

local function getIsSupportetValue(supportetValues, supportetValue)
	for supportetValueNumber = 1, #supportetValues do
		if supportetValues[supportetValueNumber] == supportetValue then 
			return true;
		end;
	end;
	
	return false;
end;

function TrailerTurnLightDashboardFix.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(Drivable, specializations);
end;

function TrailerTurnLightDashboardFix.registerEventListeners(vehicleType)
	local functionNames = {
		"onLoad",
		"onUpdate"
	};
	
	for _, functionName in ipairs(functionNames) do
		SpecializationUtil.registerEventListener(vehicleType, functionName, TrailerTurnLightDashboardFix);
	end;
end;

function TrailerTurnLightDashboardFix:onLoad(savegame)
	local specTrailerTurnLightDashboardFix = getSpecByName(self, "trailerTurnLightDashboardFix");

	specTrailerTurnLightDashboardFix.trailerTurnLights = {};

	local trailerTurnLightNumber = 0;

	while true do
		local  trailerTurnLightKey = "vehicle.trailerTurnLights.trailerTurnLight(" .. tostring(trailerTurnLightNumber) .. ")";
		
		if not self.xmlFile:hasProperty(trailerTurnLightKey) then
			break;
		end;
		
		local trailerTurnLight = {};
		
		trailerTurnLight.node = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile.handle, trailerTurnLightKey .. "#node"), self.i3dMappings);
		
		if trailerTurnLight.node ~= nil then	
			trailerTurnLight.name = string.upper(Utils.getNoNil(getXMLString(self.xmlFile.handle, trailerTurnLightKey .. "#name"), TrailerTurnLightDashboardFix.supportetTrailerTurnLights[1]));

			if getIsSupportetValue(TrailerTurnLightDashboardFix.supportetTrailerTurnLights, trailerTurnLight.name) then
				trailerTurnLight.emitColor = self:getDashboardColor(self.xmlFile, Utils.getNoNil(getXMLString(self.xmlFile.handle, trailerTurnLightKey .. "#emitColor"), "GREEN"));
				trailerTurnLight.baseColor = self:getDashboardColor(self.xmlFile, Utils.getNoNil(getXMLString(self.xmlFile.handle, trailerTurnLightKey .. "#baseColor"), "BLACK"));
				trailerTurnLight.intensity = Utils.getNoNil(getXMLFloat(self.xmlFile.handle, trailerTurnLightKey .. "#intensity"), 1);
				
				setShaderParameter(trailerTurnLight.node, "lightControl", trailerTurnLight.intensity, 0, 0, 0, false);
				setShaderParameter(trailerTurnLight.node, "emitColor", trailerTurnLight.emitColor[1], trailerTurnLight.emitColor[2], trailerTurnLight.emitColor[3], 1, false);
				setShaderParameter(trailerTurnLight.node, "baseColor", trailerTurnLight.baseColor[1], trailerTurnLight.baseColor[2], trailerTurnLight.baseColor[3], 1, false);

				table.insert(specTrailerTurnLightDashboardFix.trailerTurnLights, trailerTurnLight);
			end;
		end;

		trailerTurnLightNumber = trailerTurnLightNumber + 1;
	end;
end;

function TrailerTurnLightDashboardFix:onUpdate(dt, isActiveForInput, isActiveForInputIgnoreSelection, isSelected)
	local specTrailerTurnLightDashboardFix = getSpecByName(self, "trailerTurnLightDashboardFix");
	local specAttacherJoints = getSpecByName(self, "attacherJoints");
	
	if specAttacherJoints ~= nil then
		local numberOfAttachedTrailersWithTurnLights = 0;
		
		if specAttacherJoints.attachedImplements ~= nil and #specAttacherJoints.attachedImplements > 0 then
			for implementNumber, implement in pairs(specAttacherJoints.attachedImplements) do
				--## tool 1

				if implement.object ~= nil then
					local object = implement.object;
					local object_specAttachable = getSpecByName(object, "attachable");
					local object_specLights = getSpecByName(object, "lights");
					
					if object_specLights ~= nil then
						if getHasTurnLights(object.xmlFile) then
							numberOfAttachedTrailersWithTurnLights = numberOfAttachedTrailersWithTurnLights + 1;
						end;
					end;

					local object_specAttacherJoints = getSpecByName(object, "attacherJoints");

					if object_specAttacherJoints ~= nil and object_specAttacherJoints.attachedImplements ~= nil then
						for implementNumber, implement in pairs(object_specAttacherJoints.attachedImplements) do
							--## tool 2

							if implement.object ~= nil then
								local object = implement.object;
								local object_specAttachable = getSpecByName(object, "attachable");
								local object_specLights = getSpecByName(object, "lights");
								
								if object_specLights ~= nil then
									if getHasTurnLights(object.xmlFile) then
										numberOfAttachedTrailersWithTurnLights = numberOfAttachedTrailersWithTurnLights + 1;
									end;
								end;
							end;
						end;
					end;
				end;
			end;
		end;

		if #specTrailerTurnLightDashboardFix.trailerTurnLights > 0 then
			for _, trailerTurnLight in pairs(specTrailerTurnLightDashboardFix.trailerTurnLights) do
				if self:getIsMotorStarted() then			
					setShaderParameter(trailerTurnLight.node, "lightControl", 0, 0, 0, 0, false);
				
					local intensity = 0;
	
					local turnLightDirections = {
						"ANY",
						"LEFT",
						"RIGHT"
					};
	
					for _, turnLightDirection in pairs(turnLightDirections) do
						for trailerNumber = 1, 2 do
							if trailerTurnLight.name == "TURN_LIGHT_TRAILER_" .. turnLightDirection .. "_" .. trailerNumber then
								if numberOfAttachedTrailersWithTurnLights >= trailerNumber then		
									if turnLightDirection == "ANY" then	
										if self.spec_lights.turnLightState ~= Lights.TURNLIGHT_OFF then
											intensity = trailerTurnLight.intensity;
										end;
									else
										if self.spec_lights.turnLightState == Lights["TURNLIGHT_" .. turnLightDirection] or self.spec_lights.turnLightState == Lights.TURNLIGHT_HAZARD then
											intensity = trailerTurnLight.intensity;
										end;
									end;
								end;
							
								setShaderParameter(trailerTurnLight.node, "lightControl", intensity, 0, 0, 0, false);
							end;
						end;
					end;
				else	
					if getHasClassId(trailerTurnLight.node, ClassIds.SHAPE) and getHasShaderParameter(trailerTurnLight.node, "lightControl") then
						setShaderParameter(trailerTurnLight.node, "lightControl", 0, 0, 0, 0, false);
					end;
				end;
			end;
		end;
	end;
end;